import numpy as np
import cv2
from skimage.metrics import structural_similarity as ssim
from math import log10, sqrt

def get_psnr(y_est: np.ndarray, y_ref: np.ndarray) -> float:
    """
    Return PSNR value for y_est and y_ref presuming the noise-free maximum is 1.
    :param y_est: Estimate array
    :param y_ref: Noise-free reference
    :return: PSNR value
    """
    
    #mse = np.mean( (y_ref - y_est)**2 )
    #if(mse == 0):
        #return 100
    
    #max_pixel = 255
    #psnr = 20*log10(max_pixel / sqrt(mse))

    #return psnr
    return 10 * np.log10(1 / np.mean(((y_est - y_ref).ravel()) ** 2))

def get_ssim(y_est, y_ref) -> float:
     metric = ssim(y_ref, y_est, data_range=y_est.max() - y_est.max())

     return metric


def STATS(num, method):
    
    count = 1

    ref_image = ("frame%d.png" % count)
    ref_img = cv2.imread(ref_image).astype(float)

    if method == 'noisy':
        est_image = (method + "frame%d.png" % count)
        est_img = cv2.imread(est_image).astype(float)
    else:
        est_image = (method + "_denoisedframe%d.png" % count)
        est_img = cv2.imread(est_image).astype(float)

    psnr_single = get_psnr(est_img, ref_img)
    #struct_sim = get_ssim(est_img, ref_img)
    psnr_avg = np.empty(num)
    psnr_avg[1] = psnr_single

    count += 1

    while count <= num:
        
        if method == 'noisy':
            est_image = (method + "frame%d.png" % count)
            est_img = cv2.imread(est_image).astype(float)
        else:
            est_image = (method + "_denoisedframe%d.png" % count)
            est_img = cv2.imread(est_image).astype(float)
        
        index = count-1
        psnr_avg[index] = get_psnr(est_img, ref_img)

        count += 1

    avg = np.mean(psnr_avg)

    return psnr_single, avg