"""
This code splits up the loaded video into frames
"""

import numpy as np
import cv2
import os
from fspecial import fspecial_gaussian_2d
from bilateral import bilateral2d
from nlm import nonlocalmeans
from bm3d import bm3d_rgb, bm3d_deblurring, bm3d
from skimage.filters import gaussian
from scipy.ndimage import median_filter, gaussian_filter

# Function to extract frames 
def FrameCapture(filename): 

    vidObj = cv2.VideoCapture(filename) 

    num_frames = 26

    if vidObj.isOpened():
        count = 1
        actual_count = 1
        
        while True:
            
            success, image = vidObj.read()
 
            if success:
                
                if count%5 ==0:
                    cv2.imwrite("frame%d.png" % actual_count, image)
                    actual_count += 1
                    count += 1
                else:
                    count += 1
            #count += 1
            
            #vidObj.set(cv2.CAP_PROP_POS_FRAMES, (fps*delay))

            if count >= num_frames:
                break

    vidObj.release()
    

    actual_count -= 1
    print("The number of frames extracted was: ", actual_count)

    return actual_count
  
  # Video Generating function 
def generate_video(num, method): 
    print("Generating ", method, " video...")
    
    count = 1
    video_name = method + "_ver_video.mp4"

    # Read first frame for dimensions
    check = (method + "_denoisedframe%d.png" % count)
    img = cv2.imread(check).astype(float)
    dim = img.shape
    width, height = dim[0], dim[1]

    # Create video object
    fourcc = cv2.VideoWriter_fourcc(*'XVID')
    video = cv2.VideoWriter(video_name, fourcc, 5, (width, height))

    while count <= num:
        check = (method + "_denoisedframe%d.png" % count)
        img = cv2.imread(check)

        video.write(img)
        cv2.imshow('one frame', img)
        count += 1

    # Deallocating memories taken for window creation   
    video.release()  # releasing the video generated
    cv2.destroyAllWindows()



def AddNoise(sigma, num):
    count = 1

    while count <= num:
        
        image_name = ("frame%d.png" % count)

        #img = io.imread(image_name, as_gray=True).astype(float)
        #img = io.imread(image_name).astype(float)
        #img = cv2.imread(image_name, cv2.IMREAD_GRAYSCALE).astype(float)
        img = cv2.imread(image_name, cv2.IMREAD_GRAYSCALE).astype(float)/255
        #io.imshow(img)

        #print("Image Data:\n", img)
        img_size = img.shape
        #print(img_size)

        # NOISE ADDITION FROM PAPER
        #noise_type = 'gw'
        #noise_var = sigma
        #seed = 0
        #noise, psd, kernel = get_experiment_noise(noise_type, noise_var, seed, img.shape)
        #noisy = img + noise

        # Caelia's GAUSSIAN NOISE ADDITION
        noise = np.random.normal(0, sigma, img.shape)
        # Add to image
        noisy = img + noise
        # Clip values to ensure they are in the valid range [0, 1]
        noisy = np.clip(noisy, 0, 1)
        # Convert to 8-bit and save the noisy image
        noisy = (noisy * 255).astype(np.uint8)
        #io.imshow(noisy)

        cv2.imwrite("noisyframe%d.png" % count, noisy)
        
        left = (num - count)
        print("Frames left: ", left)
        count += 1

    #return c
        
#BM3D denoising
def Execute_BM3D(num, sigma):
    count = 1

    #print("We're going innnnn")
    while count <= num:

        image_name = ("noisyframe%d.png" % count)
        img = cv2.imread(image_name).astype(float)

        #img_deblur = bm3d_deblurring(img, sigma, psf)
        print("Starting estimation ", count)
        img_est = bm3d(img, sigma)

        cv2.imwrite("bm3d_denoisedframe%d.png" % count, img_est)

        count += 1


#Gaussian denoising
def Execute_Gauss(num, sigma):
    count = 1
    #filtSize = 2*sigma + 1

    while count <= num:

        image_name = ("noisyframe%d.png" % count)
        #img = cv2.imread(image_name, cv2.IMREAD_GRAYSCALE).astype(float)
        img = cv2.imread(image_name).astype(float)

        #c = fspecial_gaussian_2d(img.shape, sigma)
        print("Starting estimation ", count)
        img_est = gaussian_filter(img, sigma=sigma)
        #img_est = convolve2d(img, c, mode='same')
        
        cv2.imwrite("gauss_denoisedframe%d.png" % count, img_est)

        count += 1


#Bilateral filtering denoising
def Execute_Bilateral(num, sigma):
    count = 1
    #filtSize = 2*sigma + 1

    while count <= num:

        image_name = ("noisyframe%d.png" % count)
        #img = cv2.imread(image_name, cv2.IMREAD_GRAYSCALE).astype(float)
        img = cv2.imread(image_name).astype(float)

        sigmaIntensity = 0.25
        print("Starting estimation ", count)
        #img_est = cv2.bilateralFilter(img, 3, sigma, sigma)
        img_est = bilateral2d(img, radius=int(sigma), sigma=sigma, sigmaIntensity=sigmaIntensity)
        
        cv2.imwrite("bilateral_denoisedframe%d.png" % count, img_est)

        count += 1


#NonLocal Means Algorithm
def Execute_NonLocal(num, sigma):
    count = 1

    nlmSigma = 0.1  # Feel free to modify
    searchWindowRadius = 5
    averageFilterRadius = int(sigma)

    while count <= num:

        image_name = ("noisyframe%d.png" % count)
        img = cv2.imread(image_name).astype(float)

        print("Starting estimation ", count)
        img_est = np.zeros_like(img)
        for channel in [0, 1, 2]:
            img_est[channel:channel+1] = nonlocalmeans(img[channel:channel+1], searchWindowRadius, averageFilterRadius, sigma, nlmSigma)
        
        #img_est = nonlocalmeans(img, searchWindowRadius, averageFilterRadius, sigma, nlmSigma)
        cv2.imwrite("nonlocal_denoisedframe%d.png" % count, img_est)

        count += 1